%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This version: July 2013
%
% Note: This code replicates column 3 (Data-dep. prior) of Table 3 and 4 
% of the paper 
%
% Magnus, J.R. and W. Wang "Concept-based Bayesian model averaging and
% growth empirics", Oxford Bulletin of Economics and Statistics, 76:874-897
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



clc;
clear all;

%% --- Read data
data         = xlsread('D2');                             % read data

%% --- Normalization
for i = 3:length(data(1,:))                                                 
    data(:,i) = (data(:,i)-mean(data(:,i)))./std(data(:,i));
end
data(:,4)    = -data(:,4);
data(:,6)    = -data(:,6);                                                  
data(:,22)   = -data(:,22);
data(:,28)   = -data(:,28);
data(:,35)   = -data(:,35);

%% --- Dependent/independent variables
y            = data(:,1);                                                   % dependent variable
const        = ones(length(data(:,1)),1);                                   % constant term
data1        = data(:,3:4);                                                 % demographic characteristics	
data2        = data(:,5:6);                                                 % economy system
data3        = data(:,7:15);                                                % education		
data4        = data(:,16:20);                                               % government intervention
data5        = data(:,21:22);                                               % health
data6        = data(:,23:24);                                               % inflation
data7        = data(:,25:26);                                               % initial state
data8        = data(:,27:28);                                               % democracy
data9        = data(:,29:30);                                               % scale effect	
data10       = data(:,31:32);                                               % trade policy indics
data11       = data(:,33:35);                                               % tropic zone effect
data12       = data(:,36:37);                                               % war
data13       = data(:,38:44);                                               % regional effect
data14       = data(:,45);                                                  % religion
data15       = data(:,46);                                                  % ethnicity and Language		
data16       = data(:,47:49);                                               % natural resource
data17       = data(:,50);                                                  % trade statistics
data18       = data(:,51:56);                                               % population
data19       = data(:,57:59);                                               % geography excluding tropic 
data20       = data(:,60);                                                  % investment price
data21       = data(:,61:66);                                               % others

regor        = [const data1(:,1) data2(:,1) data3(:,1) data4(:,1) ...,
                data5(:,1) data6(:,1) data7(:,1) data8(:,1) data9(:,1) ...,
                data10(:,1) data11(:,1) data12(:,1) data13 data14 ...,
                data15 data16 data17 data18 data19 data20 data21];          % regressor

%% --- Number of observations and regressors
n            = length(data(:,1));                                           % observations
k            = length(regor(1,:));                                          % number of regressors

M(1)         = length(data1(1,:));   M(2)         = length(data2(1,:));     % number of measurement for tyep I group                                      
M(3)         = length(data3(1,:));   M(4)         = length(data4(1,:));                                          
M(5)         = length(data5(1,:));   M(6)         = length(data6(1,:));                                          
M(7)         = length(data7(1,:));   M(8)         = length(data8(1,:));                                           
M(9)         = length(data9(1,:));   M(10)        = length(data10(1,:));                                         
M(11)        = length(data11(1,:));  M(12)        = length(data12(1,:));                                          

MT = M(1)*M(2)*M(3)*M(4)*M(5)*M(6)*M(7)*M(8)*M(9)*M(10)*M(11)*M(12);        % total number of possible combinations  

%% --- Choose focus and auxiliary variables
k1            = 8;                                                          % focus
k2            = k-k1;                                                       % auxiliary

%% --- Set the priori probability 
for i = 1:12                                                                % assign equal probability in a group
    for j = 1:M(i)
        p(j,i) = 1/M(i);
    end
end

%% --- Update the prior, posterior probability
pp  = zeros(size(p));
 for i = 1:12
     likelihood       = zeros(1,length(p(:,i)));
     tempd            = zeros(1,M(i));
     for j = 1:M(i)                                                         % compute likelihood for each measurement
             eval(['tempX = [data14 data15 data20 data', num2str(i),'(:,',num2str(j),')]'';']);
             [tempb tempstat] = robustfit(tempX',y);
             tempsigma        = tempstat.ols_s;
             tempresid        = tempstat.resid;
             tempd(j)         = sum(((tempresid)./(tempsigma)).^2);
             likelihood(j)    = 1/(sqrt(2*pi)*tempsigma).^n.*exp(-tempd(j)./2);
             clear tempb tempstat;
     end
     for j = 1:M(i)                                                         % update the prior probability
         pp(j,i)  = p(j,i)*likelihood(j)./( sum(p(:,i).*likelihood') );  
     end
 end

% %% --- Update the prior, posterior probability (two steps)
% pp  = zeros(size(p));
%  for i = [3,5,7,11]
%      likelihood       = zeros(1,length(p(:,i)));
%      tempd            = zeros(1,M(i));
%      for j = 1:M(i)                                                         % compute likelihood for each measurement
%              eval(['tempX = [data14 data15 data20 data', num2str(i),'(:,',num2str(j),')]'';']);
%              [tempb tempstat] = robustfit(tempX',y);
%              tempsigma        = tempstat.ols_s;
%              tempresid        = tempstat.resid;
%              tempd(j)         = sum(((tempresid)./(tempsigma)).^2);
%              likelihood(j)    = 1/(sqrt(2*pi)*tempsigma).^n.*exp(-tempd(j)./2);
%              clear tempb tempstat;
%      end
%      for j = 1:M(i)                                                         % update the prior probability
%          pp(j,i)  = p(j,i)*likelihood(j)./( sum(p(:,i).*likelihood') );  
%      end
%      eval(['[maxv,mm',num2str(i),'] = max(pp(:,',num2str(i),'));']);        % record the measurement with larger posterior probability
%  end
%  
%  tempFX = [data3(:,mm3) data5(:,mm5) data7(:,mm7) data11(:,mm11) data15 data14 data20];
%  for i = [1,2,4,6,8,9,10,12]
%      for j = 1:M(i)                                                         % compute likelihood for each measurement
%          eval(['tempX = [tempFX data', num2str(i),'(:,',num2str(j),')]'';']);
%          [tempb tempstat] = robustfit(tempX',y);
%          tempsigma        = tempstat.ols_s;
%          tempresid        = tempstat.resid;
%          tempd(j)         = sum(((tempresid)./(tempsigma)).^2);
%          likelihood(j)    = 1/(sqrt(2*pi)*tempsigma).^n.*exp(-tempd(j)./2);
%          tempsigmaa(j) = tempsigma;
%          clear tempb tempstat;
%      end
%      for j = 1:M(i)                                                         % update the prior probability
%          pp(j,i)  = p(j,i)*likelihood(j)./( sum(p(:,i).*likelihood') );  
%      end
%      
%  end
%  
 
%% --- Compute estimator for all possible combination
upm     = zeros(k,1);
upv1    = zeros(k,1);
upv     = zeros(k,1);
for i1 = 1:M(1)                                                             
    for i2 = 1:M(2)
        for i3 = 1:M(3)
            for i4 = 1:M(4)
                for i5 = 1:M(5)
                    for i6 = 1:M(6)
                        for i7 = 1:M(7)
                            for i8 = 1:M(8)
                                for i9 = 1:M(9)
                                    for i10 = 1:M(10)
                                        for i11 = 1:M(11)
                                            for i12 = 1:M(12)
prob        = ...,                                                          % probability
    pp(i1,1)*pp(i2,2)*pp(i3,3)*pp(i4,4)*pp(i5,5)*pp(i6,6)*pp(i7,7) ...,
    *pp(i8,8)*pp(i9,9)*pp(i10,10)*pp(i11,11)*pp(i12,12);
D           = ...,                                                          % construct data for regression
    [y const data3(:,i3) data5(:,i5) data7(:,i7) data11(:,i11) data15 ...,
     data14 data20 data1(:,i1) data2(:,i2) data4(:,i4)  data6(:,i6) ...,
    data8(:,i8) data9(:,i9) data10(:,i10)  data12(:,i12) data17 data13 ...,
    data16 data18 data19 data21];                                    
[b, se]     = feval(['est_wals'], D, n, k1, k2);                            % obtain WALS estimators

%% Unconditional posterior mean
upm         = prob.* b + upm;                                  

%% Component for unconditional posterior variance
vb          = se.^2;                                                        
upv1        = prob.*(vb + diag(b*b')) + upv1;
                                         
                                           
                                            end
                                        end
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end
end

%% Unconditional posterior variance
upv     =  upv1 - diag(upm*upm');  
upsd=sqrt(upv);

%% t statistics
upt     = upm./upsd;
